#!/bin/bash

LOG_FILE=/var/log/plex_uninstall.log
HOST_DATA_DIR_LIBRARY=/path/to/library
HOST_DATA_DIR_TVSERIES=/path/to/tvseries
HOST_DATA_DIR_MOVIES=/path/to/movies
CONTAINER_NAME=plex
APP_VERSION=1.32.1.6999-91e1e2e2c-ls167
APP_IMAGE=lscr.io/linuxserver/$CONTAINER_NAME:$APP_VERSION

preCheck() {
    
    if ! command -v tee > /dev/null; then
        echo "`date` tee command not found. Please install tee." >> $LOG_FILE
        exit 1
    fi
    
    if ! command -v docker-compose > /dev/null; then
        echo "`date` docker-compose command not found. Please install docker-compose." | tee -a $LOG_FILE
        exit 1
    fi
    
    if ! command -v docker > /dev/null; then
        echo "`date` docker command not found. Please install docker." | tee -a $LOG_FILE
        exit 1
    fi
    return 0
}

uninstall() {
    # Stop plex
    docker-compose down
    echo "`date` wait 10 seconds for $CONTAINER_NAME to stop" | tee -a $LOG_FILE
    sleep 10
    
    # Clear docker images
    docker rmi $APP_IMAGE
    
    # Clear data
    rm -rf $HOST_DATA_DIR_LIBRARY
    rm -rf $HOST_DATA_DIR_TVSERIES
    rm -rf $HOST_DATA_DIR_MOVIES
    return 0
}

postCheck() {
    echo "`date` start post check for $CONTAINER_NAME uninstallation" | tee -a $LOG_FILE
    
    # Check plex status
    if docker-compose ps | grep $CONTAINER_NAME | grep Up > /dev/null; then
        echo "`date` plex is running." | tee -a $LOG_FILE
        return 1
    fi

    # Clear docker images
    if docker images | grep $CONTAINER_NAME | grep 5.7 > /dev/null; then
        echo "`date` plex image is not removed." | tee -a $LOG_FILE
        return 1
    fi


    if [ -d "$HOST_DATA_DIR_LIBRARY" ]; then
        echo "`date` plex library data is not removed." | tee -a $LOG_FILE
        return 1
    fi

    if [ -d "$HOST_DATA_DIR_TVSERIES" ]; then
        echo "`date` plex tvseries data is not removed." | tee -a $LOG_FILE
        return 1
    fi

    if [ -d "$HOST_DATA_DIR_MOVIES" ]; then
        echo "`date` plex movies data is not removed." | tee -a $LOG_FILE
        return 1
    fi
    
    return 0
}

# step 1: pre-Check
preCheck
preCheckResult=$?
echo "`date` pre check result is $preCheckResult" | tee -a $LOG_FILE
if [ $preCheckResult -eq 0 ]
then
    echo "`date` Pre-check successful." | tee -a $LOG_FILE
    # Step 2: Uninstall plex
    uninstall
    uninstallResult=$?
    if [ $uninstallResult -eq 0 ]
    then
        echo "`date` Uninstall plex successful." | tee -a $LOG_FILE
        # Step 3: Post-check
        postCheck
        postCheckResult=$?
        if [ $postCheckResult -eq 0 ]
        then
            echo "`date` Post-check successful." | tee -a $LOG_FILE
            echo "`date` plex uninstallation process completed." | tee -a $LOG_FILE
            exit 0
        else
            echo "`date` Post-check failed." | tee -a $LOG_FILE
            exit 9
        fi
    else
        echo "`date` Uninstall plex failed." | tee -a $LOG_FILE
        exit 8
    fi
else
    echo "`date` Pre-check failed." | tee -a $LOG_FILE
    exit 7
fi