#!/bin/bash

LOG_FILE=/var/log/plex_upgrade.log
HOST_DATA_DIR_LIBRARY=/path/to/library
HOST_DATA_DIR_TVSERIES=/path/to/tvseries
HOST_DATA_DIR_MOVIES=/path/to/movies
CONTAINER_NAME=plex
APP_VERSION=1.32.1.6999-91e1e2e2c-ls167
APP_IMAGE=lscr.io/linuxserver/$CONTAINER_NAME:$APP_VERSION

preCheck() {
    
    if ! command -v tee > /dev/null; then
        echo "`date` tee command not found. Please install tee." >> $LOG_FILE
        exit 1
    fi
    
    if ! command -v docker-compose > /dev/null; then
        echo "`date` docker-compose command not found. Please install docker-compose." | tee -a $LOG_FILE
        exit 1
    fi
    
    if [ -z "$1" ]; then
        echo "`date` Parameter port is missing or empty." | tee -a $LOG_FILE
        exit 1
    fi
    
    # Check Plex status
    if docker ps -a | grep $CONTAINER_NAME | grep Up > /dev/null; then
        docker stop $CONTAINER_NAME
        stopResult=$?
        if [ $stopResult -ne 0 ]; then
            echo "`date` stop $CONTAINER_NAME failed with exit code $stopResult" | tee -a $LOG_FILE
            return 1
        fi
        docker rm $CONTAINER_NAME
        rmResult=$?
        if [ $rmResult -ne 0 ]; then
            echo "`date` rm $CONTAINER_NAME failed with exit code $rmResult" | tee -a $LOG_FILE
            return 1
        fi
    fi

    if docker ps -a | grep $CONTAINER_NAME | grep Exited > /dev/null; then
        docker rm $CONTAINER_NAME
        rmResult=$?
        if [ $rmResult -ne 0 ]; then
            echo "`date` rm $CONTAINER_NAME failed with exit code $rmResult" | tee -a $LOG_FILE
            return 1
        fi
    fi
    
    return 0
}

upgrade() {
    
    PORT=$1
    echo "`date` start upgrade $CONTAINER_NAME on port $PORT" | tee -a $LOG_FILE
    # Create docker-compose.yml
    echo "version: '2.1'" > docker-compose.yml
    echo "services:" >> docker-compose.yml
    echo "  $CONTAINER_NAME:" >> docker-compose.yml
    echo "    image: $APP_IMAGE" >> docker-compose.yml
    echo "    container_name: $CONTAINER_NAME" >> docker-compose.yml
    echo "    restart: unless-stopped" >> docker-compose.yml
    echo "    ports:" >> docker-compose.yml
    echo "      - $PORT:32400" >> docker-compose.yml
    echo "    environment:" >> docker-compose.yml
    echo "      - PUID=1000" >> docker-compose.yml
    echo "      - PGID=1000" >> docker-compose.yml
    echo "      - TZ=Etc/UTC" >> docker-compose.yml
    echo "      - VERSION=docker" >> docker-compose.yml
    echo "      - PLEX_CLAIM=claim-dUJW8w2xw4Ytso4UjX5k" >> docker-compose.yml
    echo "    volumes:" >> docker-compose.yml
    echo "      - $HOST_DATA_DIR_LIBRARY:/config" >> docker-compose.yml
    echo "      - $HOST_DATA_DIR_TVSERIES:/tv" >> docker-compose.yml
    echo "      - $HOST_DATA_DIR_MOVIES:/movies" >> docker-compose.yml
    
    # Start Plex
    docker-compose up -d
    upResult=$?
    if [ $upResult -ne 0 ]; then
        echo "`date` upgrade $CONTAINER_NAME failed with exit code $upResult" | tee -a $LOG_FILE
        return 1
    fi
    echo "`date` wait 10 seconds for $CONTAINER_NAME to start" | tee -a $LOG_FILE
    sleep 10
    
    return 0
}

postCheck() {
    echo "`date` start post check for $CONTAINER_NAME upgrade" | tee -a $LOG_FILE
    # Check Plex status
    if ! docker-compose ps | grep $CONTAINER_NAME | grep Up > /dev/null; then
        echo "`date` Plex is not running." | tee -a $LOG_FILE
        return 1
    fi
    return 0
}

# step 1: pre-Check
preCheck $1
preCheckResult=$?
echo "`date` pre check result is $preCheckResult" | tee -a $LOG_FILE
if [ $preCheckResult -eq 0 ]
then
    echo "`date` Pre-check successful." | tee -a $LOG_FILE
    # Step 2: Upgrade Plex
    upgrade $1
    upgradeResult=$?
    if [ $upgradeResult -eq 0 ]
    then
        echo "`date` Upgrade Plex successful." | tee -a $LOG_FILE
        # Step 3: Post-check
        postCheck
        postCheckResult=$?
        if [ $postCheckResult -eq 0 ]
        then
            echo "`date` Post-check successful." | tee -a $LOG_FILE
            echo "`date` Plex upgrade process completed." | tee -a $LOG_FILE
            exit 0
        else
            echo "`date` Post-check failed." | tee -a $LOG_FILE
            exit 6
        fi
    else
        echo "`date` Upgrade Plex failed." | tee -a $LOG_FILE
        exit 5
    fi
else
    echo "`date` Pre-check failed." | tee -a $LOG_FILE
    exit 4
fi